from modules.ply_reader import *
from modules.hough_transform import *
from modules.helper_scripts import *
import hdbscan  # conda install -c conda-forge hdbscan
from sklearn.cluster import OPTICS, DBSCAN, AgglomerativeClustering


def plot(img, lines, points, colors, classes):
    cv2.imshow("a", img)
    cv2.waitKey()
    for line in lines:
        x1, y1, x2, y2 = line[0]
        cv2.line(img, (x1, y1), (x2, y2), (255, 0, 0), 3)

    for idx in range(len(points)):
        x, y, _ = points[idx]

        if classes[idx] == -1:
            color = (0, 0, 0)
        else:
            color = hex_to_rgb(colors[classes[idx]])

        cv2.circle(img, (int(x), int(y)), radius=3, color=color, thickness=-1)

    # Show result
    cv2.imshow("Result Image", img)
    cv2.waitKey()  # cv2.imshow won't work without this :)
    cv2.destroyAllWindows()


def HDBSCAN(data):
    clusterer = hdbscan.HDBSCAN(algorithm='best', alpha=1.0, approx_min_span_tree=True,
                                gen_min_span_tree=False, leaf_size=40,
                                metric='euclidean', min_cluster_size=2, min_samples=4, p=None)
    clusterer.fit(data)
    return clusterer.labels_


def optics(data):
    clusterer = OPTICS(min_samples=2,
                       max_eps=100,
                       metric='minkowski',
                       p=2,
                       cluster_method='xi',
                       xi=0.05,
                       predecessor_correction=True,
                       algorithm='auto')
    clusterer.fit(data)
    return clusterer.labels_


def dbscan(data):
    clusterer = DBSCAN(eps=0.5,
                       min_samples=2,
                       metric='euclidean',
                       max_eps=100)
    clusterer.fit(data)
    return clusterer.labels_


def agglomerativeClustering(data):
    clusterer = AgglomerativeClustering(n_clusters=None,
                                        affinity='euclidean')
    clusterer.fit(data)
    return clusterer.labels_


if __name__ == '__main__':
    directory = '/Users/aigaandrijanova/Documents/GitHub/alarm_pointcloud_backend_2021_Q4/data/'

    for file in os.listdir(directory):
        filename = os.fsdecode(file)
        if filename.endswith(".ply"):

            path = os.path.join(directory, filename)
            vertices = PlyReader.read(path)
            img = PlyReader.points_3D_to_image(vertices, show=False)
            img = HoughTransform.rotate_image(img, show=True)

            lines = HoughTransform.detect_lines(img, max_slider=30)
            data, points = lines_to_midpoints_and_angles(lines)

            labels = HDBSCAN(data)
            colors = generate_colours(labels.max() + 1)
            plot(img, lines, points, colors, labels)

            labels = optics(data)
            colors = generate_colours(labels.max() + 1)
            plot(img, lines, points, colors, labels)
        else:
            continue
